from ldappy.tools.toolbox import Toolbox
from ldappy.tools.configuration import Configuration

from base64 import urlsafe_b64decode as b64d
from typing import Optional
from dataclasses import dataclass, field
import zlib
import logging
import ldap3

logger = logging.getLogger(__name__)


@dataclass
class Ldapclient:
    conn: ldap3.Connection = field(init=False, repr=False)

    def __post_init__(self):
        self.conn = self.__class__._init_ldap_connection()
        logger.debug(self.conn)

    @staticmethod
    def _init_ldap_connection():
        """Connection au ldap
        """
        config = Configuration.loadldap()

        server_uri = config.get('HOST')
        port = config.get('PORT')
        base: Optional[str] = config.get('BASE')
        ldap_user = zlib.decompress(b64d(config.get('LDAP_USER'))).decode('utf-8')
        ldap_pass: Optional[str] = zlib.decompress(b64d(config.get('LDAP_PASSWORD'))).decode('utf-8')

        server = ldap3.Server(server_uri, get_info=ldap3.ALL)
        return ldap3.Connection(server, source_port=int(port), user=ldap_user, password=ldap_pass, auto_bind=True)

