from typing import List, Optional

from pydantic import BaseModel, constr

class Employee(BaseModel):
    """ Information de base d'un utilisateur """

    prenom: str
    nom: str
    mail: constr(regex=r'.*@univ-reunion.fr$')
    sihamid: constr(regex=r'RUN\d{9}')
    authorisation: Optional[List[str]]

    def data_values(self) -> List[str]:
        """ Récupère les valeurs des champs de données de l'employée.

        Returns:
            Une liste qui contient les données de l'employée.
        """
        return list(self.dict().values())

    def data_keys(self) -> List[str]:
        """ Récupère la liste des champs de l'employée.

        Returns:
            Une liste des champs de l'employée.
        """
        return list(self.dict().keys())

    # def display(self) -> None:
    #     """ Affiche les informations de l'employée sous forme de tableau"""
    #     print(tabulate([self.data_values()], headers=self.data_keys(), tablefmt='pipe'))

    def is_existing_authorization(self, elm: str) -> bool:
        """ Check si une chaine de caractère existe dans l'attribut "authorisation".

        Args:
            elm (str): La chaine de caractère recherché.

        Returns:
            True si l'élément est dans authorisation.
            False si l'élément n'y est pas.
        """
        if elm in self.authorisation:
            return True
        return False
